/*
 * pnlConcreteValueConstraint.java
 *
 * Created on 18. januar 2006, 16:40
 */

package jcreek.gui.representation.constraints;

import javax.swing.*;
import jcreek.representation.*;
import jcreek.representation.constraints.*;
import jcreek.util.EntityNameComparator;
import java.util.*;
import java.awt.Dimension;
/**
 *
 * @author  Martin Stige
 *
 * Panel to show a form for defining the allowed values
 * for Concrete Value Constraints.<br>
 *
 * Constains a ComboBox with all the entities in the knowledge model, a JList
 * for showing the choosen entities, and two JButtons, one to add entities and one
 * to remove an entity from the list.
 */
public class pnlConcreteValueConstraint extends javax.swing.JPanel {
    private KnowledgeModel km;
    private ListModel valueListModel;
    private ArrayList values;
    
    /** Creates new form pnlConcreteValueConstraint */
    public pnlConcreteValueConstraint() {
        initComponents();
        values = new ArrayList();
    }
    
    /** Gives the panel a reference to the knowledge model. From the knowledge
     * model it can get all the entities it wants
     *
     * @param km the knowledge model kontaining the entities to be shown in
     * the combobox
     */
    public void setKnowledgeModel(KnowledgeModel km){
        this.km = km;
        initMyComponents();
    }
    
    /* Initiates the components that are not made by the GUI-editor */
    private void initMyComponents(){
        initComboBox();
        initList();
        
    }
    
    /** Initiates the combobox */
    private void initComboBox(){
        Entity[] rcds = km.getEntities();
        Arrays.sort(rcds, new EntityNameComparator());
        Object[] choices = new Object[rcds.length + 1];
        choices[0] = "<Choose entity>";
        System.arraycopy(rcds, 0, choices, 1, rcds.length);
        cboEntities.setModel(new DefaultComboBoxModel(choices));
        cboEntities.setSelectedIndex(0);
        //cboEntities.setMaximumSize(new Dimension(300, 19));
        //cboEntities.setPreferredSize(new Dimension(200, 19));
        
        
    }
    
    /** Initiates the entity list  */
    private void initList(){
        valueListModel = new AbstractListModel() {
            public int getSize() { return values.size(); }
            public Object getElementAt(int index) { return values.get(index); }
        };
        
        lstValues.setModel(valueListModel);
    }
    
    
    /**
     * Returns the a list containing the entities in the list
     */
    public ArrayList getValues(){ return values; }
    
    /**
     * Returns whether the values in the list are allowed or not
     */
    public boolean getAllowed(){ return rdbAllowed.isSelected(); }
    
    /**
     * Clears the form
     */
    public void clear(){
        values = new ArrayList();
        cboEntities.setSelectedIndex(0);
    }
    
    /**
     * Sets the allowed-value
     * @param allowed true = allowed, false = disallowed
     */
    public void setAllowed(boolean allowed){
        rdbAllowed.setSelected(allowed);
    }
    
    /**
     * Sets the list of values (entities)
     * @param values the list of entities either allowed or disallowed
     */
    public void setValues(ArrayList values){
        this.values = values;
        initList();
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        groupAllowed = new javax.swing.ButtonGroup();
        lblHeader = new javax.swing.JLabel();
        pnlCenter = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        lstValues = new javax.swing.JList();
        btnDelete = new javax.swing.JButton();
        pnlSouth = new javax.swing.JPanel();
        cboEntities = new jcreek.gui.representation.constraints.HorizontalScrollCombo();
        btnAdd = new javax.swing.JButton();
        pnlAllowed = new javax.swing.JPanel();
        rdbAllowed = new javax.swing.JRadioButton();
        rdbDisallowed = new javax.swing.JRadioButton();

        setLayout(new java.awt.BorderLayout());

        setMinimumSize(new java.awt.Dimension(300, 61));
        setPreferredSize(new java.awt.Dimension(300, 150));
        lblHeader.setText("Allowed values");
        add(lblHeader, java.awt.BorderLayout.NORTH);

        pnlCenter.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setPreferredSize(new java.awt.Dimension(200, 70));
        jScrollPane1.setViewportView(lstValues);

        pnlCenter.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        btnDelete.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcreek/gui/resources/delete.gif")));
        btnDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDeleteActionPerformed(evt);
            }
        });

        pnlCenter.add(btnDelete, java.awt.BorderLayout.EAST);

        add(pnlCenter, java.awt.BorderLayout.CENTER);

        pnlSouth.setLayout(new java.awt.BorderLayout());

        cboEntities.setBorder(new javax.swing.border.EtchedBorder());
        cboEntities.setMaximumSize(new java.awt.Dimension(200, 32767));
        cboEntities.setPreferredSize(new java.awt.Dimension(220, 21));
        pnlSouth.add(cboEntities, java.awt.BorderLayout.CENTER);

        btnAdd.setText("Add");
        btnAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAddActionPerformed(evt);
            }
        });

        pnlSouth.add(btnAdd, java.awt.BorderLayout.EAST);

        pnlAllowed.setLayout(new java.awt.GridLayout());

        rdbAllowed.setSelected(true);
        rdbAllowed.setText("Values allowed");
        groupAllowed.add(rdbAllowed);
        rdbAllowed.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                rdbAllowedStateChanged(evt);
            }
        });

        pnlAllowed.add(rdbAllowed);

        rdbDisallowed.setText("Values dissalowed");
        groupAllowed.add(rdbDisallowed);
        pnlAllowed.add(rdbDisallowed);

        pnlSouth.add(pnlAllowed, java.awt.BorderLayout.SOUTH);

        add(pnlSouth, java.awt.BorderLayout.SOUTH);

    }//GEN-END:initComponents
    
    private void rdbAllowedStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_rdbAllowedStateChanged
        if(rdbAllowed.isSelected()){
            lblHeader.setText("Allowed values");
        }else{
            lblHeader.setText("Disallowed values");
        }
    }//GEN-LAST:event_rdbAllowedStateChanged
    
    private void btnDeleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDeleteActionPerformed
        values.remove(lstValues.getSelectedIndex());
        lstValues.repaint();
    }//GEN-LAST:event_btnDeleteActionPerformed
    
    private void btnAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddActionPerformed
        if(cboEntities.getSelectedIndex() == 0)
            return;
        else{
            values.add(cboEntities.getSelectedItem());
            initList();
        }
    }//GEN-LAST:event_btnAddActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAdd;
    private javax.swing.JButton btnDelete;
    private jcreek.gui.representation.constraints.HorizontalScrollCombo cboEntities;
    private javax.swing.ButtonGroup groupAllowed;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lblHeader;
    private javax.swing.JList lstValues;
    private javax.swing.JPanel pnlAllowed;
    private javax.swing.JPanel pnlCenter;
    private javax.swing.JPanel pnlSouth;
    private javax.swing.JRadioButton rdbAllowed;
    private javax.swing.JRadioButton rdbDisallowed;
    // End of variables declaration//GEN-END:variables
    
    
}

