/*
 * pnlConstraintLine.java
 *
 * Created on 13. januar 2006, 14:29
 */

package jcreek.gui.representation.constraints;

import jcreek.representation.*;
import jcreek.util.EntityNameComparator;
import java.util.Arrays;
import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

import jcreek.representation.constraints.*;
/**
 *
 * @author  Martin Stige
 *
 * Internal panel for the stuff that differ between the constraint types
 */
public class pnlConstraintLine extends javax.swing.JPanel {
    private KnowledgeModel km;
    private Entity myEntity;
    private Constraint selectedConstraint = null;
    
    /** Creates new form pnlConstraintLine */
    public pnlConstraintLine() {
        initComponents();
        //initMyComponents();
        
    }
    
    /** Initiates the components that are not handeled by the GUI-editor */
    private void initMyComponents() {
        if(myEntity == null)
            return;
        km = myEntity.getKnowledgeModel();
        
        Entity[] rcds = km.getRelationType("Relation").getAllSubclasses();
        
        for(int i = 0; i < rcds.length; i++){
            if(!(rcds[i] instanceof RelationType))
                try{
                    rcds[i] = new RelationType(rcds[i]);
                }catch(EntityTypeNotMatchedException e){
                    e.printStackTrace();
                }
        }
        Arrays.sort(rcds, new EntityNameComparator());
        Object[] choices = new Object[rcds.length + 1];
        choices[0] = "<Choose relation>";
        System.arraycopy(rcds, 0, choices, 1, rcds.length);
        cboRelations.setModel(new DefaultComboBoxModel(choices));
        cboRelations.setSelectedIndex(0);
        
        
        
        cboConstraint.setModel(new javax.swing.DefaultComboBoxModel(
        new String[] {
            "<Choose constraint>",      //  0
            "Cardinality constraint",
            "Value range constraint",
            "Value class constraint",
            "Concrete value constraint",
            "General constraint"}  //  1
        ));
        
    }
    
    /**
     * Sets the entity that shall be showed and manipulated
     * @param e the entity
     */
    public void setEntity(Entity e) {
        myEntity = e;
        initMyComponents();
        if(e!=null){
            pnlValueClassConstraint.setKnowledgeModel(e.getKnowledgeModel());
            pnlConcreteValueConstraint.setKnowledgeModel(e.getKnowledgeModel());
            pnlGeneralConstraint.setEntity(e);
            clear();
        }
        
        
    }
    /** Clears the form */
    public void clear(){
        hideAllConstraintPanels();
        
        txtDescription.setText("<Description of the constraints>");
        pnlCardinalityConstraint.clear();
        pnlConcreteValueConstraint.clear();
        pnlValueClassConstraint.clear();
        pnlValueRangeConstraint.clear();
        pnlGeneralConstraint.clear();
        cboRelations.setSelectedIndex(0);
        cboConstraint.setSelectedIndex(0);
        remove(cboRelations);
        validate();
    }
    
    /** Sets the constraint to be showed */
    public void setConstraint(Constraint c){
        selectedConstraint = c;
        
        if(c instanceof CardinalityConstraint){
            txtDescription.setText(c.getDescription());
            CardinalityConstraint cc = (CardinalityConstraint)c;
            cboConstraint.setSelectedIndex(1);
            
            cboRelations.setSelectedItem(cc.getRelation());
            pnlCardinalityConstraint.setMax(cc.getMax());
            pnlCardinalityConstraint.setMin(cc.getMin());
        }
        else if(c instanceof ValueRangeConstraint){
            txtDescription.setText(c.getDescription());
            ValueRangeConstraint vrc = (ValueRangeConstraint)c;
            cboConstraint.setSelectedIndex(2);
            cboRelations.setSelectedItem(c.getRelation());
            
            pnlValueRangeConstraint.setMax(vrc.getMax());
            pnlValueRangeConstraint.setMin(vrc.getMin());
            pnlValueRangeConstraint.setAllowed(vrc.getAllowed());
        }
        else if(c instanceof ValueClassConstraint){
            txtDescription.setText(c.getDescription());
            ValueClassConstraint vcc = (ValueClassConstraint)c;
            cboConstraint.setSelectedIndex(3);
            cboRelations.setSelectedItem(c.getRelation());
            
            pnlValueClassConstraint.setEntity(vcc.getSuperEntity());
        }
        else if(c instanceof ConcreteValueConstraint){
            txtDescription.setText(c.getDescription());
            ConcreteValueConstraint cvc = (ConcreteValueConstraint)c;
            cboConstraint.setSelectedIndex(4);
            cboRelations.setSelectedItem(c.getRelation());
            
            pnlConcreteValueConstraint.setAllowed(cvc.getAllowed());
            pnlConcreteValueConstraint.setValues(cvc.getValues());
        }
        else if(c instanceof GeneralConstraint){
            txtDescription.setText(c.getDescription());
            GeneralConstraint gc = (GeneralConstraint)c;
            cboConstraint.setSelectedIndex(5);
            
            pnlGeneralConstraint.setConstraintString(gc.getConstraintString());
        }
        
        
    }
    
    /** Puts changes if the form into the constraint object */
    public void updateConstraint(){
        if(selectedConstraint == null)
            return;
        if((cboConstraint.getSelectedIndex() + cboRelations.getSelectedIndex()) < 2)
            return;
        
        selectedConstraint.setDescription(txtDescription.getText());
        
        if(selectedConstraint instanceof GeneralConstraint){
            System.out.println("updating general constraint");
            GeneralConstraint c = (GeneralConstraint)selectedConstraint;
            c.setConstraintString(pnlGeneralConstraint.getConstraintString());
        }
        else if(selectedConstraint instanceof CardinalityConstraint){
            selectedConstraint.setRelation((RelationType) cboRelations.getSelectedItem());
            CardinalityConstraint c = (CardinalityConstraint)selectedConstraint;
            c.setMax(pnlCardinalityConstraint.getMax());
            c.setMin(pnlCardinalityConstraint.getMin());
        }
        else if(selectedConstraint instanceof ValueRangeConstraint){
            selectedConstraint.setRelation((RelationType) cboRelations.getSelectedItem());
            ValueRangeConstraint c = (ValueRangeConstraint)selectedConstraint;
            c.setMax(pnlValueRangeConstraint.getMax());
            c.setMin(pnlValueRangeConstraint.getMin());
            c.setAllowed(pnlValueRangeConstraint.getAllowed());
        }
        else if(selectedConstraint instanceof ValueClassConstraint){
            selectedConstraint.setRelation((RelationType) cboRelations.getSelectedItem());
            ValueClassConstraint c = (ValueClassConstraint)selectedConstraint;
            c.setSuperEntity(pnlValueClassConstraint.getEntity());
        }
        else if(selectedConstraint instanceof ConcreteValueConstraint){
            selectedConstraint.setRelation((RelationType) cboRelations.getSelectedItem());
            ConcreteValueConstraint c = (ConcreteValueConstraint)selectedConstraint;
            c.setValues(pnlConcreteValueConstraint.getValues());
            c.setAllowed(pnlConcreteValueConstraint.getAllowed());
        }
        
        
    }
    
    /** Creates a constraint object from the form and returns it*/
    public Constraint getConstraint(){
        if(cboConstraint.getSelectedItem().equals("Cardinality constraint"))
            return makeCardinalityConstraint();
        else if(cboConstraint.getSelectedItem().equals("Value range constraint"))
            return makeValueRangeConstraint();
        else if(cboConstraint.getSelectedItem().equals("Value class constraint"))
            return makeValueClassConstraint();
        else if(cboConstraint.getSelectedItem().equals("Concrete value constraint"))
            return makeConcreteValueConstraint();
        else if(cboConstraint.getSelectedItem().equals("General constraint"))
            return makeGeneralConstraint();
        return null;
    }
    /** Makes a cardinality constraint */
    private CardinalityConstraint makeCardinalityConstraint(){
        int min;
        int max;
        
        try{
            min = pnlCardinalityConstraint.getMin() ;
            max = pnlCardinalityConstraint.getMax();
            RelationType type = (RelationType)cboRelations.getSelectedItem();
            
            return new CardinalityConstraint(km, type, min, max, txtDescription.getText());
        }catch(NumberFormatException e){
            e.printStackTrace();
            return null;
        }catch(Exception e){
            e.printStackTrace();
            return null;
        }
    }
    
    /** Makes a value range constraint */
    private ValueRangeConstraint makeValueRangeConstraint(){
        try{
            Number min = pnlValueRangeConstraint.getMin() ;
            Number max = pnlValueRangeConstraint.getMax();
            boolean allowed = pnlValueRangeConstraint.getAllowed();
            RelationType type = (RelationType)cboRelations.getSelectedItem();
            
            return new ValueRangeConstraint(km, type, min, max, allowed, txtDescription.getText());
        }catch(Exception e){
            e.printStackTrace();
            return null;
        }
    }
    
    /** Makes a value class constraint */
    private ValueClassConstraint makeValueClassConstraint(){
        try{
            Entity superEntity = pnlValueClassConstraint.getEntity();
            RelationType type = (RelationType)cboRelations.getSelectedItem();
            
            return new ValueClassConstraint(km, type, superEntity, txtDescription.getText());
        }catch(Exception e){
            e.printStackTrace();
            return null;
        }
    }
    
    /** Makes a concrete value constraint */
    private ConcreteValueConstraint makeConcreteValueConstraint(){
        RelationType type = (RelationType)cboRelations.getSelectedItem();
        ArrayList values = pnlConcreteValueConstraint.getValues();
        boolean allowed = pnlConcreteValueConstraint.getAllowed();
        
        return new ConcreteValueConstraint(km, type, values,allowed, txtDescription.getText());
    }
    
    /** Makes a general constraint */
    private GeneralConstraint makeGeneralConstraint(){
        return new GeneralConstraint(km, pnlGeneralConstraint.getConstraintString(), txtDescription.getText());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        pnlValueRangeConstraint = new jcreek.gui.representation.constraints.pnlValueRangeConstraint();
        pnlValueClassConstraint = new jcreek.gui.representation.constraints.pnlValueClassConstraint();
        pnlConcreteValueConstraint = new jcreek.gui.representation.constraints.pnlConcreteValueConstraint();
        pnlCardinalityConstraint = new jcreek.gui.representation.constraints.pnlCardinalityConstraint();
        pnlGeneralConstraint = new jcreek.gui.representation.constraints.pnlGeneralConstraint();
        txtDescription = new javax.swing.JTextField();
        cboConstraint = new jcreek.gui.representation.constraints.HorizontalScrollCombo();
        cboRelations = new jcreek.gui.representation.constraints.HorizontalScrollCombo();

        pnlValueRangeConstraint.setPreferredSize(new java.awt.Dimension(290, 46));
        pnlValueClassConstraint.setPreferredSize(new java.awt.Dimension(290, 21));
        pnlConcreteValueConstraint.setPreferredSize(new java.awt.Dimension(290, 150));
        pnlCardinalityConstraint.setPreferredSize(new java.awt.Dimension(290, 20));
        pnlGeneralConstraint.setPreferredSize(new java.awt.Dimension(290, 175));

        setBorder(new javax.swing.border.MatteBorder(null));
        setMinimumSize(new java.awt.Dimension(250, 100));
        setPreferredSize(new java.awt.Dimension(300, 260));
        txtDescription.setText("<Description of the constraints>");
        txtDescription.setPreferredSize(new java.awt.Dimension(290, 21));
        add(txtDescription);

        cboConstraint.setPreferredSize(new java.awt.Dimension(290, 21));
        cboConstraint.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cboConstraintActionPerformed(evt);
            }
        });

        add(cboConstraint);

        cboRelations.setPreferredSize(new java.awt.Dimension(290, 21));
        add(cboRelations);

    }//GEN-END:initComponents
    
    /** Update constraint line to show a form that fits the selecte constraint type */
    private void cboConstraintActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cboConstraintActionPerformed
        hideAllConstraintPanels();
        
        if(cboConstraint.getSelectedItem().equals("Cardinality constraint"))
            add(pnlCardinalityConstraint);
        else if(cboConstraint.getSelectedItem().equals("Value range constraint"))
            add( pnlValueRangeConstraint);
        else if(cboConstraint.getSelectedItem().equals("Value class constraint"))
            add(pnlValueClassConstraint);
        else if(cboConstraint.getSelectedItem().equals("Concrete value constraint"))
            add(pnlConcreteValueConstraint);
        else if(cboConstraint.getSelectedItem().equals("General constraint")){
            add(pnlGeneralConstraint);
            remove(cboRelations);
        }
        
        validate();
        
    }//GEN-LAST:event_cboConstraintActionPerformed
    
    /** Hides all constraint panels*/
    private void hideAllConstraintPanels(){
        add(cboRelations);
        remove(pnlCardinalityConstraint);
        remove(pnlValueRangeConstraint);
        remove(pnlValueClassConstraint);
        remove(pnlConcreteValueConstraint);
        remove(pnlGeneralConstraint);
        repaint();
        validate();
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private jcreek.gui.representation.constraints.HorizontalScrollCombo cboConstraint;
    private jcreek.gui.representation.constraints.HorizontalScrollCombo cboRelations;
    private jcreek.gui.representation.constraints.pnlCardinalityConstraint pnlCardinalityConstraint;
    private jcreek.gui.representation.constraints.pnlConcreteValueConstraint pnlConcreteValueConstraint;
    private jcreek.gui.representation.constraints.pnlGeneralConstraint pnlGeneralConstraint;
    private jcreek.gui.representation.constraints.pnlValueClassConstraint pnlValueClassConstraint;
    private jcreek.gui.representation.constraints.pnlValueRangeConstraint pnlValueRangeConstraint;
    private javax.swing.JTextField txtDescription;
    // End of variables declaration//GEN-END:variables
    
}
