/*
 * CardinalityConstraint.java
 *
 * Created on 12. januar 2006, 11:50
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.constraints.*;

/**
 * Offers the ability from define cardinality constraints for the relations
 * of a given type for some entity. Two integer values, max and min defines
 * the maximum and minimum cardinality.
*
 * @author  Martin Stige
 */
public class CardinalityConstraint extends Constraint {
    
    /**
     * Creates a new instance of CardinalityConstraint
     *
     * @param knowledgeModel the knowledge model
     * @param rel the relation to be constrained
     * @param numberMin the minimum cardinality
     * @param numberMax the maximum cardinality
     * @param description description of the constraint
     */
    public CardinalityConstraint(KnowledgeModel knowledgeModel, RelationType rel, int numberMin, int numberMax, String description) {
        super(knowledgeModel, description);
        constraintData.setConstraintObject( new CardinalityConstraintObject(rel, numberMin, numberMax) );
    }
    
    /**
     * Creates a new instance of CardinalityConstraint from a constraint data object
     * @param knowledgeModel the knowledge model
     * @param constraintData the constraint data to be contained in the constraint
     */
    public CardinalityConstraint(KnowledgeModel knowledgeModel, ConstraintData constraintData){
        super(knowledgeModel, constraintData);
    }
    
    /**
     * Sets the minimum cardinality value
     * @param min the minimum cardinality
     */
    public void setMin(int min){
        CardinalityConstraintObject c = (CardinalityConstraintObject)(constraintData.getConstraintObject());
        c.setMin(min);
    }
    
    /**
     * Sets the maximum cardinality value
     * @param max the maximum cardinality
     */
    public void setMax(int max){
        CardinalityConstraintObject c = (CardinalityConstraintObject)(constraintData.getConstraintObject());
        c.setMax(max);
    }
    
    /**
     * Returns the minimum cardinality constraint
     */
    public int getMin(){
        CardinalityConstraintObject c = (CardinalityConstraintObject)(constraintData.getConstraintObject());
        return c.getMin();
    }
    
    /**
     * Returns the maximum cardinality constraint
     */
    public int getMax(){
        CardinalityConstraintObject c = (CardinalityConstraintObject)(constraintData.getConstraintObject());
        return c.getMax();
    }
}

