/*
 * ConcreteValueConstraint.java
 *
 * Created on 12. januar 2006, 11:55
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.constraints.*;
import java.util.ArrayList;

/**
 * Offers the ability to define constraints based on concrete values.
 * You can either define the closed set of allowed values, or define
 * a set of values that are not allowed.
 * <p>
 * It is not interresting to define both these two constraints on the same
 * entity->relation. If the (positive) allowed is used, all values not
 * on the list are implicitly defined as disallowed. <br>
 * If the (negative) disallowed is used all values not on the list are allowed.
 *
 * @author  Martin Stige
 */
public class ConcreteValueConstraint  extends Constraint{
    
    /**
     * Creates a new instance of ConcreteValueConstraint
     *
     * @param knowledgeModel the knowledge model the constraint shall bellong to
     * @param rel the relation type the constraint shall constrain
     * @param values the concrete values to be allowed or not
     * @param allowed whether the values shall be allowed or not
     * @param description a description of the constraint
     */
    public ConcreteValueConstraint(KnowledgeModel knowledgeModel, RelationType rel, ArrayList values, boolean allowed, String description) {
        super(knowledgeModel, description);
        constraintData.setConstraintObject( new ConcreteValueConstraintObject(rel, values, allowed) );
    }
     
    /**
     * Creates a new instance of ConcreteValueConstraint
     *
     * @param knowledgeModel the knowledge model the constraint shall bellong to
     * @param constraintData the constraint data to be contained by the constraint
     */
    public ConcreteValueConstraint(KnowledgeModel knowledgeModel, ConstraintData constraintData){
        super(knowledgeModel, constraintData);
    }
    
    /**
     * Sets the values that are either allowed or disallowed
     * @param al the list of values
     */
    public void setValues(ArrayList al){
        ConcreteValueConstraintObject co = (ConcreteValueConstraintObject)constraintData.getConstraintObject();
        co.setValues(al);
    }
    
    /**
     * Sets whether the values are allowed or disallowed
     * @param b allowed or not
     */
    public void setAllowed(boolean b){
        ConcreteValueConstraintObject co = (ConcreteValueConstraintObject)constraintData.getConstraintObject();
        co.setAllowed(b);
    }
    
    /**
     * Returns the list of allowed or disallowed values
     */
    public ArrayList getValues(){
        ConcreteValueConstraintObject co = (ConcreteValueConstraintObject)constraintData.getConstraintObject();
        return co.getValues();
    }
    
    /**
     * Returns whether the values are allowed or disallowed
     */
    public boolean getAllowed(){
        ConcreteValueConstraintObject co = (ConcreteValueConstraintObject)constraintData.getConstraintObject();
        return co.getAllowed();
    }
}
