/*
 * ConstraintCheckResult.java
 *
 * Created on 2. februar 2006, 13:59
 */

package jcreek.representation.constraints;

/**
 * This object wraps the information returned by the constraints 
 * @author  Martin Stige
 * @version 1.0
 */
public class ConstraintCheckResult {
    private boolean ok;
    private String description;
    
    /** 
     * Creates a new instance of ConstraintCheckResult 
     * @param ok whether the constraint are met or not
     * @param description an explanation of the constraint violation
     */
    public ConstraintCheckResult(boolean ok, String description) {
        this.ok = ok;
        this.description = description;
    }
    
    /** No-argument constructor */
    public ConstraintCheckResult() {}
    
    /** 
     * Sets the explanation of the constraint check
     * @param desc the explanation
     */
    public void setDescription(String desc){ description = desc; }
    
    /**
     * Sets whether the constraint is violated or not
     * @param viol true for violated, false for constraint met
     */
    public void setViolated(boolean viol){ok = !viol; }
    
    /**
     * Returns true if the constraint is violated
     */
    public boolean isViolated(){ return !ok; }
    
    /**
     * Returns the explanation of the constraint check
     */
    public String getDescription(){ return description; }
    
    public String toString(){
        return (ok? "Constraint met ": "Constraint violated ") + "\n" + description;
    }
}
