/*
 * ConstraintData.java
 *
 * Created on 5. januar 2006, 14:16
 */

package jcreek.representation.constraints;

import jcreek.representation.*;

/**
 * Interface for Constraint data.<br>
 * This interface can be implemented differently depending on how the
 * constrains(and the rest for the knowledge model shall be stored.<br>
 * Known implementations:
 * {@link  jcreek.representation.constraints.LocalConstraintData  LocalConstraintData}
 *
 * @version 1.0
 * @author  Martin Stige
 */
public interface ConstraintData {
    
    /**
     * Sets the constraint object of the constraint
     * @param constraintObject the constraint object to set for the constraint data
     */
    public void setConstraintObject(ConstraintObject constraintObject);
    
    /**
     * Returns the constraint object
     */
    public ConstraintObject getConstraintObject();
    
    /**
     * Returns the description of the constraint
     */
    public String getDescription();
    
    /**
     * Sets the description of the constraint
     * @param description the description
     */
    public void setDescription(String description);
    
    /**
     * returns the ID of the constraint
     */
    public Integer getID();
    
    public String toString();
    
    /**
     * returns an instance of the constraint
     */
    public Constraint newConstraintInstance();
    
}
