/*
 * Constraint.java
 *
 * Created on 12. oktober 2005, 12:26
 */

package jcreek.representation.constraints;

import jcreek.representation.Entity;
import jcreek.representation.RelationType;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.constraints.*;

import java.io.Serializable;

/**
 * Super class for the constraint object classes. <br>
 * Constraint object is the object describing the behaviour of the constraints.
 * The behaviour of the constraint is defined by the check method
 *
 * @author  Martin Stige
 */
public abstract class ConstraintObject implements Serializable {
    protected transient KnowledgeModel km;
    protected String rel;
    
    /** Checks  whether the constraint is violated or not.
     * <p>
     * Should be augmented to offer some feedback about the violations. Until
     * that is made, feedback is just printed to System.out.
     *
     * @return true if everyting is ok, false if the constraint is violated
     */
    public abstract ConstraintCheckResult check(Entity from);
    
    /** Creates a Constraint object of a specific type
     * @param km the knowledge model that the constraint shall bellong to
     * @param data the ConstraintData object to be contained by the constriant
     */
    public abstract Constraint newConstraintInstance(KnowledgeModel km, ConstraintData data);
    
    public abstract String toString();
    
    /** Sets the relation type the constraint shall constrain
     * @param relType the relation type
     */
    public void setRelation(RelationType relType) { rel = relType.getName(); }
    
    /** Sets the knowledge model the constraint shal bellong to
     * @param km the KnowledgeModel
     */
    public void setKnowledgeModel(KnowledgeModel km){ this.km = km; }
    
    /** Returns the relation type that the constraint constrains
     * @return a RelationType
     */
    public RelationType getRelation() { return km.getRelationType(rel); }
}
