/*
 * GeneralConstraintObject.java
 *
 * Created on 13. februar 2006, 12:10
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.constraints.ConstraintCheckResult;
/**
 * Handles the behaviour of general constraints.<p>
 * Offers the ability to define arbitrary complex constraints written i Java.
 * @author  Martin Stige
 */
public class GeneralConstraintObject extends ConstraintObject{
    private String constraintString;
    private transient DJavaAdapter interpreter;
    
    /**
     * Creates a new instance of GeneralConstraintObject
     * @param constraintString the source code string
     */
    public GeneralConstraintObject(String constraintString) {
        this.constraintString = constraintString;
    }
    
    /**
     * Checks the constraint on an entity
     * @param from the entity to check with this constraint
     */
    public ConstraintCheckResult check(Entity from){
        interpreter = new DJavaAdapter(from);
        return interpreter.validate(constraintString);
        
    }

    /** 
     * Creates a new constraint object of this type 
     * @param km the knowledge model
     * @param data the constraint data object to be contained in the constraint
     */
    public Constraint newConstraintInstance(KnowledgeModel km, ConstraintData data) {
        return new GeneralConstraint(km, data);
    }
    
    /** Returs the constraint string */
    protected String getConstraintString(){ return constraintString; }
    
    /** 
     * Sets the constraint string (source code)
     * @param constraintString the source code
     */
    protected void setConstraintString(String constraintString){
        this.constraintString = constraintString;;
    }
    
    public String toString(){ return constraintString.substring(0, 50); }
}
