/*
 * LocalConstraintData.java
 *
 * Created on 5. januar 2006, 14:17
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import java.io.Serializable;
/**
 * Implementation of constraint data that for storing the model as a local file.<br>
 * For use together with LocalKnowledgeModel, LocalEntityData and so on.
 * @author  Martin Stige
 */
public class LocalConstraintData implements ConstraintData, Serializable {
    private transient KnowledgeModel km;
    private Integer id;
    private String description;
    private ConstraintObject constraintObject;
    
    /**
     * Creates a new instance of LocalConstraintData
     *
     * @param km the knowledge model
     * @param id the id of this constraint shall have
     */
    public LocalConstraintData(KnowledgeModel km, Integer id) {
        this.km = km;
        this.id = id;
    }
    
    /** Returns the description of the constraint */
    public String getDescription() {
        return description;
    }
    
    /**
     * Sets the description of the constraint data
     * @param description
     */
    public void setDescription(String description) {
        this.description = description;
    }
    
    /** Returns the constraint object */
    public ConstraintObject getConstraintObject() {
        return constraintObject;
    }
    
    /**
     * Sets constraint object
     * @param constraintObject the constraintObject
     */
    public void setConstraintObject(ConstraintObject constraintObject) {
        this.constraintObject = constraintObject;
        constraintObject.setKnowledgeModel(km);
    }
    
    /** Returns the ID of the constraint */
    public Integer getID(){ return id; }
    
    public String toString(){ return constraintObject.toString();}
    
    /** Returns an instance of the right constraint type */
    public Constraint newConstraintInstance() {
        return constraintObject.newConstraintInstance(km, this);
    }
    
    /**
     * Sets the knowledge model <br>
     * This method is used for building up the structures in the knowledge model
     * during loading from file. If you scroll some centimeters up, you probaboy
     * understand why - the km variable in this class is declared transient. 
     * 
     * @param km the knowledge model
     */
    public void setKnowledgeModel(KnowledgeModel km){
        this.km = km;
        constraintObject.setKnowledgeModel(km);
    }
   
}
