/*
 * ValueClassConstraint.java
 *
 * Created on 12. januar 2006, 11:52
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
/**
 * Offers the ability to state that the value of the relations of a given
 * type from a given entity must be subclasses of some class/entity.
 * <p>
 * It is probably a good idea to make a negative counterpart of this one, ore
 * include the negative value class in this constraint. But that is defered...
 *
 * @author  Martin Stige
 */
public class ValueClassConstraint extends Constraint {
    
    /**
     * Creates a new instance of ValueClassConstraint
     *
     * @param knowledgeModel the knowledge model the constraint shall bellong to
     * @param rel the relation type the constraint shall constrain
     * @param superEntity the entity that the relation's value must be a subclass of
     * @param description a description of the constraint
     */
    public ValueClassConstraint(KnowledgeModel knowledgeModel, RelationType rel, Entity superEntity, String description) {
        super(knowledgeModel, description);
        
        constraintData.setConstraintObject( new ValueClassConstraintObject(rel, superEntity) );
    }
    
    /**
     * Creates a new instance of ValueClassConstraint from a constraint data object
     *
     * @param knowledgeModel the knowledge model the constraint shall bellong to
     * @param constraintData the constraint data to be contained by the constraint
     */
    public ValueClassConstraint(KnowledgeModel knowledgeModel, ConstraintData constraintData){
        super(knowledgeModel, constraintData);
    }
    
    /**
     * Sets the mandatory super entity
     * @param superEntity the entity that the relation's value must be a subclass of
     */
    public void setSuperEntity(Entity superEntity){
        ValueClassConstraintObject co = (ValueClassConstraintObject)constraintData.getConstraintObject();
        co.setSuperEntity(superEntity);
    }
    
    /**
     * Returns the super entity
     */
    public Entity getSuperEntity(){
        ValueClassConstraintObject co = (ValueClassConstraintObject)constraintData.getConstraintObject();
        return co.getSuperEntity();
    }
}
