/*
 * Constraint.java
 *
 * Created on 12. oktober 2005, 10:12
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
import jcreek.representation.cbr.*;
import jcreek.representation.constraints.ConstraintCheckResult;

/**
 * Handles the behaviour of Value class constraints.<p>
 * Offers the ability to state that the value of the relations of a given
 * type from a given entity must be subclasses of some class/entity.
 * <p>
 * It is probably a good idea to make a negative counterpart of this one, ore
 * include the negative value class in this constraint. But that is defered...
 *
 * @author  Martin Stige
  */
public class ValueClassConstraintObject extends ConstraintObject {
    private Object supercl;
    
    /** Creates a new instance of Value Class Constraint<br>
     * from must be a subclass of supercl
     *
     * @param rel the relation being restricted
     * @param supercl the entity that target of the relation must be an instance of
     */
    public ValueClassConstraintObject(RelationType rel, Entity supercl) {
        this.supercl = supercl.getID();
        this.rel = rel.getName();
    }
    
    public ConstraintCheckResult check(Entity from){
        Relation[] rels = from.getRelations(getRelation());
        String[] errors = new String[rels.length];
        int errorIndex = 0;
        
        for(int i = 0; i < rels.length; i ++){
            Relation r = rels[i];
            Entity e = r.getValue();
            
            Entity[] superclasses = e.getAllSuperclasses();
            boolean tmp = false;
            
            for(int j = 0; j < superclasses.length; j++){
                Entity e1 = superclasses[j];
                if(e1.equals(km.getEntity(supercl))){
                    tmp = true;
                    j = superclasses.length;
                }
            }
            
            if(!tmp)
                errors[errorIndex++] = from.getName() + " -> " + r.getName() + " -> " + e.getName();
        }
        
        String result = "";
        if(errorIndex > 0){
            result += "Value class allowed " + km.getEntity(supercl).getName() + "\n";
            
            for(int i = 0; i < errorIndex; i++)
                result += errors[i] +"\n";
            return new ConstraintCheckResult(false, result);
        }else
            return new ConstraintCheckResult(true, result);
        
    }
    
    protected Entity getSuperEntity() { return km.getEntity(supercl); }
    
    protected void setSuperEntity(Entity e){ supercl = e.getID(); }
    
    public Constraint newConstraintInstance(KnowledgeModel km, ConstraintData data) {
        return new ValueClassConstraint(km, data);
    }
    
    public String toString(){
        String ret = "Value class(";
        // ret += from.getName() + ", ";
        ret += getRelation().getName() + ", ";
        ret += km.getEntity(supercl).getName();
        ret += ")";
        
        return ret;
    }
    
}
