/*
 * ValueRangeConsraint.java
 *
 * Created on 12. januar 2006, 11:54
 */

package jcreek.representation.constraints;

import jcreek.representation.*;
/**
 * Offers the ability to constrain the value of nummerical slots.
 * You can either define an allowed range of values or a range that is not allowed
 *
 * @author  Martin Stige
 */
public class ValueRangeConstraint extends Constraint{
    
    /** Creates a new instance of ValueRangeConsraint */
    public ValueRangeConstraint(KnowledgeModel knowledgeModel, RelationType rel, Number numberMin, Number numberMax, boolean allowed, String description) {
        super(knowledgeModel, description);
        
        constraintData.setConstraintObject( new ValueRangeConstraintObject(rel, numberMin, numberMax, allowed) );
    }
    
    public ValueRangeConstraint(KnowledgeModel knowledgeModel, ConstraintData constraintData){
        super(knowledgeModel, constraintData);
    }
    
    /**
     * Sets whether the value range shall be the allowed values
     * or disallowed values.
     * @param b allowed or not
     */
    public void setAllowed(boolean b){ getObject().setAllowed(b); }
    
    /**
     * Returns whether the range are allowed or not
     */
    public boolean getAllowed(){ return getObject().getAllowed(); }
    
    /**
     * Sets the minimum number of the range
     * @param min the minimum value
     */
    public void setMin(Number min){ getObject().setMin(min); }
    
    /**
     * Returns the minimum number of the range
     */
    public Number getMin(){ return getObject().getMin(); }
    
    /**
     * Sets the maximum number of the range
     * @param max the maximum value
     */
    public void setMax(Number max){ getObject().setMax(max); }
    
    /**
     * Returns the maximum number of the range
     */
    public Number getMax(){ return getObject().getMax(); }
    
    
    /**
     * Casts the ConstraintObject to the right type
     */
    private ValueRangeConstraintObject getObject(){
        return (ValueRangeConstraintObject) constraintData.getConstraintObject();
    }
}
